/***************************************************************************
 *   Copyright (C) 2005-2009 by Hannes Hauswedell                          *
 *   kports@soulrebel.in-berlin.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __PORTSCOLLECTION_H__
#define __PORTSCOLLECTION_H__

#include <QObject>
#include "bsdport.h"


class QPortsCollection : public QObject
{
Q_OBJECT
private:
    QPortList m_collection;

//     mutable
public:
    enum SEARCHFLAGS
    {
        PKGNAME = 1, CATEGORIES = 2, MAINT = 4, COMMENT = 8,
        DESCR = 16, FILES = 32
    };

    enum PKGTYPE
    {
        AVSRCPKG = 1, AVBINPKG = 2, INSTPKG = 4,
        ANYPKG = AVSRCPKG + AVBINPKG + INSTPKG
    };
    
    QPortsCollection(QObject* parent = 0)
        : QObject(parent)
    {
    }

//     QPortsCollection(const QPortsCollection& old)
//         : QObject(old.parent())
//     {
//         m_collection = old.m_collection;
//     }
    
    ~QPortsCollection()
    {
        qDeleteAll(m_collection);
    }

    inline const QPortList & collection() const
    {
        return m_collection;
    }
    inline QBsdPort* at(ulong index) const
    {
        if (index >= m_collection.size())
            return 0;
        else
            return m_collection.at(index);
    }

    inline void append(QBsdPort* newValue)
    {
        m_collection.append(newValue);
        emit newPortAdded(newValue);
    }

    
    inline void insertSorted(QBsdPort* newValue)
    {
        int i = 0;
        while ( newValue->origin() > at(i)->origin()
                && i < m_collection.size())
            ++i;
        if (i < m_collection.size())
            if (at(i)->origin() == newValue->origin())
            {
                qDebug("port already in list: "
                       + newValue->origin().toLatin1());
                return;
            }
        m_collection.insert(i, newValue);
        emit newPortAdded(newValue);
    }
    
    inline ulong size()
    {
        return m_collection.size();
    }

public slots:
    void sort()
    {
        qSort(m_collection.begin(),
              m_collection.end(),
              QBsdPort::lessThan);
    }

    void removeDuplicates()
    {
        // check for ports mistakenly added twice
        QBsdPort* oldp = 0;
        foreach(QBsdPort* p, m_collection)
        {
            if (oldp)
            {
                if (oldp == p) //same object added twice
                {
                    qDebug("double-added port removed");
                    m_collection.removeOne(p);
                }

                if (oldp->origin() == p->origin()) // two objects for one port
                {
                    qDebug("double-referenced port removed " +
                           p->origin().toLatin1());
                    
                    m_collection.removeOne(p);
                    p->deleteLater();
                    //recheck stuff on oldp
                    if (!oldp->hasInst())
                        oldp->findInst();
                    if (!oldp->hasAvSrc())
                        oldp->findAvSrc();
                    if (!oldp->hasAvBin())
                        oldp->findAvBin();
                }
            }
            oldp = p;
        }
    }

    void clear()
    {
        emit aboutToClear();
        qDeleteAll(m_collection);
        m_collection.clear();
    }
    
    /// return port with corresponding origin
    QBsdPort    *portWithOrigin(const  QString& origin,
                                uint   startPos = 0,
                                int    *returnPos = 0 ) const;
                                
    /// return port with corresponding name (and other vars)
    const QPortList   portWithPkgName(const QString& Name,
                                      PKGTYPE    pkgtype = ANYPKG,
                                      bool withVersion = true) const;

    QPortList search(const QString& term,
                     int criteria,
                     int    pkgtype = ANYPKG,
                     bool exact = false);
    QPortList search(const QString& term,
                     qint64 flags);

    inline void     addPkgToPort(const QString origin,
                                 QBsdPortPkg*   pkg)
    {
        QBsdPort *p = portWithOrigin(origin);
        if (!p)
        {
//             qDebug(QString("Not found: " + origin).toLatin1()); 
            p = new QBsdPort(origin);
            insertSorted(p);
        }
        addPkgToPort(p, pkg);
    }

    inline void     addPkgToPort(QBsdPort*      port,
                                 QBsdPortPkg*   pkg)
    {
        if (QBsdPortInstPkg *Pkg = qobject_cast<QBsdPortInstPkg *>(pkg))
            port->setInst(Pkg);
        else if (QBsdPortAvBinPkg *Pkg = qobject_cast<QBsdPortAvBinPkg *>(pkg))
            port->setAvBin(Pkg);
        else if (QBsdPortAvSrcPkg *Pkg = qobject_cast<QBsdPortAvSrcPkg *>(pkg))
            port->setAvSrc(Pkg);
    }



signals:
    void    newPortAdded(QBsdPort*);
    void    aboutToClear();
    /** concerning search **/
    void    searching();
    void    searchProgress(int);
    void    searchResult(QBsdPort*);
    void    searchFinished();
};




#endif

